import logging

from vllm import LLM
from vllm_inference.inference import inference

logger = logging.getLogger('vllm_inference')


def translate_cot(llm: LLM, sentence: str, prefix: str = '', target_lang: str = 'en', resample: int = 1, **kwargs) -> str:
    pre_think = '[COT]'
    post_think = '[COT]'
    lang_seq = f'<{target_lang.lower().replace("<", "").replace(">", "")}>'
    kwargs['stop'] = post_think
    prompt = f'Translate the sentence and explain in detail.<sentence>{sentence}</sentence>{lang_seq}{pre_think}{prefix}'
    logger.debug(f'PROMPT: {prompt.replace("\n", " ")}')
    cot = sorted(inference(prompt=[prompt] * resample, llm=llm, **kwargs),
                 key=lambda x: len(x[0]), reverse=True)[0]
    logger.debug(f'COT: {cot}')
    return cot



from vllm import LLM
from vllm.model_executor.layers.quantization import QuantizationMethods


def load_model(model_name: str, gpu_memory_utilization: float = .9,
               swap_space: float = 0, cpu_offload_gb: float = 0, max_seq_len_to_capture: int = 4096,
               quantization: QuantizationMethods = 'fp8'):
    return LLM(model=model_name, trust_remote_code=True, gpu_memory_utilization=gpu_memory_utilization,
               swap_space=swap_space, cpu_offload_gb=cpu_offload_gb, max_seq_len_to_capture=max_seq_len_to_capture,
               quantization=quantization)



import logging
import os

__ROOT__ = os.path.dirname(__file__)
os.environ['HF_HOME'] = os.path.join(__ROOT__, '.cache')
os.environ['HF_ENDPOINT'] = 'https://hf-mirror.com'

logger = logging.getLogger('vllm_inference')
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter('[%(levelname)s] %(asctime)s %(name)s : %(message)s')

console_handler = logging.StreamHandler()
console_handler.setLevel(logging.DEBUG)
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)



import logging

from vllm import LLM, TextPrompt
from vllm.model_executor.guided_decoding.guided_fields import GuidedDecodingRequest
from vllm.sampling_params import BeamSearchParams, SamplingParams

logger = logging.getLogger('vllm_inference')


def inference_with_beam_search(prompt: str, llm: LLM, temperature: float = .0, beam_width: int = 4, max_tokens: int = 4096) -> list[tuple[str, float]]:
    beam_search_params = BeamSearchParams(temperature=temperature, beam_width=beam_width, max_tokens=max_tokens, include_stop_str_in_output=False)
    results = llm.beam_search(prompts=[TextPrompt(prompt=prompt)], params=beam_search_params)[0].sequences
    results.sort(key=lambda x: x.cum_logprob, reverse=True)
    logger.debug(f'inference_with_beam_search: {results}')
    return [(res.text, res.cum_logprob) for res in results]


def inference(prompt: str | list[str], llm: LLM, regex: str | None = None, **kwargs) -> list[str]:
    prompt = [TextPrompt(prompt=prompt)] if isinstance(prompt, str) else [TextPrompt(prompt=p) for p in prompt]
    max_retry = 8
    retry = 0
    tmp_results = []
    results = []
    while retry < max_retry:
        results = llm.generate(prompt, sampling_params=SamplingParams(**kwargs),
                               guided_options_request=GuidedDecodingRequest(guided_regex=regex) if regex else None,
                               use_tqdm=True)
        logger.debug(f'inference: {results}')
        tmp_results.extend(results)
        results = [y.text for y in [x.outputs[0] for x in results] if y.finish_reason.lower() == 'stop']
        if len(results) > 0:
            return results
        retry += 1
    return results if len(results) else tmp_results
